@@ -3,7 +3,7 @@
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import If, Equal, Eval, Not, In, Get
 from trytond.transaction import Transaction
-from trytond.backend import TableHandler
+from trytond import backend
 
 __all__ = ['OrderPoint']
 
@@ -24,8 +24,7 @@ class OrderPoint(ModelSQL, ModelView):
             ('purchasable', 'in', If(Equal(Eval('type'), 'purchase'),
                     [True], [True, False])),
             ],
-        depends=['type'],
-        on_change=['product'])
+        depends=['type'])
     warehouse_location = fields.Many2One('stock.location',
         'Warehouse Location', select=True,
         domain=[('type', '=', 'warehouse')],
@@ -63,7 +62,7 @@ class OrderPoint(ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
             domain=[
                 ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
+                    Eval('context', {}).get('company', -1)),
             ])
     unit = fields.Function(fields.Many2One('product.uom', 'Unit'), 'get_unit')
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
@@ -87,6 +86,7 @@ class OrderPoint(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         # Migration from 2.2
         table = TableHandler(cursor, cls, module_name)
@@ -98,6 +98,7 @@ class OrderPoint(ModelSQL, ModelView):
     def default_type():
         return "purchase"
 
+    @fields.depends('product')
     def on_change_product(self):
         res = {
             'unit': None,
@@ -139,6 +140,7 @@ class OrderPoint(ModelSQL, ModelView):
         query = ['OR']
         for op in internals:
             arg = ['AND',
+                   ('product', '=', op.product.id),
                    ('provisioning_location', '=', op.storage_location.id),
                    ('storage_location', '=', op.provisioning_location.id),
                    ('company', '=', op.company.id),
